function par = abc2par(abc,FP)
// abc2par Convert parameters of membership functions
//
//    ANFISLAB: Toolbox de ANFIS (ADAPTATIVE-NETWORK-BASED-FUZZY INFERENCE SYSTEM) para SCILAB
//    Creado Por Andres G. Avendaño Cabezas., Universidad Central de Venezuela (andrewg.avendano@gmail.com),
//    Agosto de 2009, con adaptaciones del codigo ANFIS de dominio publico de Jyh-shing Roger Jang
//    (http://www.cs.nthu.edu.tw/~jang/publication.htm) y FISLAB de G. Ortega.
//    Este programa fue probado en SCILAB version 5.1.1, PC Intel Core(TM) Duo 2,1Ghz, corriendo Linux 2.6.27.7.
//    
//    The program was made to be "atom compatible" by Tan C.L.
//    It also modified to work seamlessly with sciFLT to provide more
//    powerful tools for neuro-fuzzy domain. 
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/
//
// Calling Sequence
//     par = abc2par(abc,FP)
//
// Parameters
//     abc : 3 parameters for membershipfunctions:  
//          - gbellmf : abc and par are the same, where f(x,a,b,c)=1/(1+abs((x-c)/a)^(2*b)).
//          - trimf : abc is triangle parameter which a = slope, b = 0, and c is the center of triangle, while par(1) is the lower left point, par(2) is the midpoint, and par(3) is the lower right point.
//          - trapmf : abc is triangle parameter which a = slope, b = flatness , and c is the center of trapezoidal, while par(1) is the lower left point, par(2) is the upper left point, par(3) is the upper right point and par(4) is lower right point. 
//     FP : Membership function, 'gbellmf', 'trimf' or 'trapmf'
//     par : Output parameters to match with sciFLT toolbox
//
// Description
//    This function convert parameters of a membership function from abc to par. This is due to the anfislab is using standardized parameters for training purposes while the sciFLT is using the user friendly parameters. Please note that this function is meant for the advance user or developer who want to adapt new membership functions for ANFIS.
// 
// Examples
//    abc = [1,0,0];
//    par = abc2par(abc,'trimf');
//    x = -2:0.1:2;
//    y = trimf(x,par);
//    plot(x,y);
// 
// See also
//    par2abc
//    fls2abcpqr
//
// Authors
//     Andres G. Avendaño Cabezas, Jyh-shing Roger Jang, Tan C.L. 
//

    
    if strcmp(FP,'gbellmf','i')==0 then
        par = abc;
    elseif strcmp(FP,'trapmf','i')==0 then
        par(:,2) = abc(:,3) - abc(:,2) ./ 2; // For Tranp
        par(:,3) = abc(:,3) + abc(:,2) ./ 2;
        par(:,1) = par(:,2)- 1 ./ abc(:,1);
        par(:,4) = 1 ./ abc(:,1) + par(:,3);
    elseif strcmp(FP,'trimf','i')==0 then
        par(:,2) = abc(:,3); // For Trianglar
        par(:,1) = abc(:,3)- 1 ./ abc(:,1);
        par(:,3) = 1 ./ abc(:,1) + abc(:,3);
    else
        error("anfislab only support gbellmf, trimf, trapmf at the moment.");
    end
    
    

endfunction
