function [out_fls,evola] = anfis(traindata,in_fls,itr,err,step_sz)
// anfis Train an ANFIS model.
//
//    ANFISLAB: Toolbox de ANFIS (ADAPTATIVE-NETWORK-BASED-FUZZY INFERENCE SYSTEM) para SCILAB
//    Creado Por Andres G. Avendaño Cabezas., Universidad Central de Venezuela (andrewg.avendano@gmail.com),
//    Agosto de 2009, con adaptaciones del codigo ANFIS de dominio publico de Jyh-shing Roger Jang
//    (http://www.cs.nthu.edu.tw/~jang/publication.htm) y FISLAB de G. Ortega.
//    Este programa fue probado en SCILAB version 5.1.1, PC Intel Core(TM) Duo 2,1Ghz, corriendo Linux 2.6.27.7.
//    
//    The program was made to be "atom compatible" by Tan C.L.
//    It also modified to work seamlessly with sciFLT to provide more
//    powerful tools for neuro-fuzzy domain. 
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/
//
// Calling Sequence
//     [out_fls,evola] = anfis(traindata,in_fls)
//     [out_fls,evola] = anfis(traindata,in_fls,itr,err,step_sz)
//
// Parameters
//     traindata : Training data, each variables arranged in a column, with last column as the output varaible
//     in_fls : ANFIS model to be trained (in format compatible with sciFLT)
//     itr : Number of iteration
//     err : Error for stoping condition
//     step_sz : Step size
//     out_fls : ANFIS model which has been trained (in format compatible with sciFLT)
//     evola : Information for each iteration
//
// Description
//    This function train the anfis model with the provided data and model.
//  
// Examples
//    x = (0:0.1:10)';
//    y = sin(2*x)./exp(x/5);
//    fls = initanfis([x y],5,'gbellmf');
//    [fls,evola] = anfis([x y],fls);
//    y2 = evalfls(x,fls);
//    plot(x,y,'bo--',x,y2,'r.--');
//    legend('Original Data', 'ANFIS Output');
// 
// See also
//    initanfis
//    editanfis
//
// Authors
//     Andres G. Avendaño Cabezas, Jyh-shing Roger Jang, Tan C.L. 
//

winId=progressionbar('Please Wait...');    
//gbellmf, trimf, trapmf
// Checking Input Arguement
rhs=argn(2);

// Error Checking
if rhs < 2; error("Expect at least 2 arguments, traindata and in_fls");end
if rhs < 3; itr = 30; end
if rhs < 4; err = 0; end        
if rhs < 5; step_sz = 0.1; end        
    
// Number of inputs data calculated automatically
pin = size(traindata,1);
k = size(traindata,2) - 1;
L = size(fls.input(1).mf);

// Initialize Training Variables
[abc,pqr]=fls2abcpqr(in_fls);

// Number of outputs data calculated automatically
valdata = [];
if valdata==[] then
    pchk=0;valdata=zeros(1,k+1)
end


    mem_func = in_fls.input(1).mf(1).type;
    if strcmp(mem_func,'gbellmf','i')==0 then
        mf_ind = 2;
    elseif strcmp(mem_func,'trapmf','i')==0 then
        mf_ind = 3;
    elseif strcmp(mem_func,'trimf','i')==0 then
        mf_ind = 4;
    else
        error("anfislab only support gbellmf, trimf, trapmf at the moment.");
    end
    
// Training
[abc,pqr,evola]=hytrain(k,L,pin,pchk,mf_ind,itr,err,step_sz,abc,pqr,traindata,valdata);
//pause
//out=check(k,L,101,mf_ind,abc,pqr,traindata);

//////////////////////////////////////////////////////////////////////
// Structure Convertion from flsLAB to SCIFLT to integrate 2 modules
//////////////////////////////////////////////////////////////////////

//if mf_ind == 1 | mf_ind == 2 then   // bell_1 and bell_2, bell_1 is currently not used
//    abc2 = abc; // For Bell2 == gbellmf
//    mem_func = 'gbellmf';
//elseif mf_ind ==3   // Trapezoidal
//    abc2(:,2) = abc(:,3) - abc(:,2) ./ 2; // For Tranp
//    abc2(:,3) = abc(:,3) + abc(:,2) ./ 2;
//    abc2(:,1) = abc2(:,2)- 1 ./ abc(:,1);
//    abc2(:,4) = 1 ./ abc(:,1) + abc2(:,3);
//    mem_func = 'trapmf';
//elseif mf_ind ==4   // Triangular
//    abc2(:,2) = abc(:,3); // For Trianglar
//    abc2(:,1) = abc(:,3)- 1 ./ abc(:,1);
//    abc2(:,3) = 1 ./ abc(:,1) + abc(:,3);
//    mem_func = 'trimf';
//end

abc2 = abc2par(abc,mem_func)

//fls=newfls();
//fls.ImpMethod = 'prod'
//fls.AggMethod = 'sum'
//fls.defuzzMethod = 'wtaver'
//

// L membership function per input
// k input variables

// Define empty list for variable and membership function
//flsvar_def=tlist(['flsvar','name','range','mf'],"",[],list());
//flsmf_def=tlist(['flsmf','name','type','par'],"","",[]);
//flsvar=flsvar_def
//flsmf=flsmf_def;
out_fls = in_fls;

k = size(out_fls.input);
L = size(out_fls.input(1).mf);

for cnt_var = 1:k
    for cnt_mf = 1:L
         out_fls.input(cnt_var).mf(cnt_mf).par = abc2((cnt_var-1)*L+cnt_mf,:);
    end
end



k2 = size(out_fls.output);
L2 = size(out_fls.output(1).mf);

clear par;
for cnt_var = 1:k2
    for cnt_mf = 1:L2
         out_fls.output(cnt_var).mf(cnt_mf).par = pqr((cnt_var-1)*L2+cnt_mf,:);
    end
end


//r1 = specfun_combinerepeat (1:L,k)';
//r2 = [1:L^k]';
//r3 = ones(L^k,2);
//out_fls.rule = [r1 r2 r3];
//
//num_outmf = size(abc,1);
//
//fls.output(1).name = 'output1';
//fls.output(1).range = [0 30];
//
//for cnt = 1:num_outmf
//    fls.output(1).mf(cnt).name = 'out1mf'+string(cnt); 
//    fls.output(1).mf(cnt).type = 'linear'; 
//    fls.output(1).mf(cnt).par =pqr(cnt,:);
//end
//
//fls.rule = [1 1 1 1;2 2 1 1;3 3 1 1;4 4 1 1; 5 5 1 1];
//
//
//
close(winId);

endfunction
