function fls = initanfis(E,L,FP);
// initanfis Create an initial ANFIS model.
//
//    ANFISLAB: Toolbox de ANFIS (ADAPTATIVE-NETWORK-BASED-FUZZY INFERENCE SYSTEM) para SCILAB
//    Creado Por Andres G. Avendaño Cabezas., Universidad Central de Venezuela (andrewg.avendano@gmail.com),
//    Agosto de 2009, con adaptaciones del codigo ANFIS de dominio publico de Jyh-shing Roger Jang
//    (http://www.cs.nthu.edu.tw/~jang/publication.htm) y FISLAB de G. Ortega.
//    Este programa fue probado en SCILAB version 5.1.1, PC Intel Core(TM) Duo 2,1Ghz, corriendo Linux 2.6.27.7.
//    
//    The program was made to be "atom compatible" by Tan C.L.
//    It also modified to work seamlessly with sciFLT to provide more
//    powerful tools for neuro-fuzzy domain. 
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/
//
// Calling Sequence
//     fls = initanfis(E,L,FP);
//
// Parameters
//     E : Training data, each variables arranged in a column, with last column as the output varaible
//     L : Number of membership functions per input variable
//     FP : Membership function, 'gbellmf', 'trimf' or 'trapmf'
//     fls : ANFIS model (in format compatible with sciFLT)
//
// Description
//    This function calculate the initial ANFIS model structure which is compatible with sciFLT module.
//    
// Examples
//    x = (0:0.1:10)';
//    y = sin(2*x)./exp(x/5);
//    fls = initanfis([x y],3,'gbellmf');
//    plotvar(fls,"input",1);
// 
// See also
//    editanfis
//    anfis
//
// Authors
//     Andres G. Avendaño Cabezas, Jyh-shing Roger Jang, Tan C.L. 
//

rhs=argn(2);

// Error Checking
if rhs < 1; error("Expect at least 1 argument, traindata");end
if rhs < 2; L = 2; end
if rhs < 3; FP = 'gbellmf'; end
mem_func = FP;

if type(FP) == 10 then   
    if strcmp(FP,'gbellmf','i')==0 then
        FP = 2;
    elseif strcmp(FP,'trapmf','i')==0 then
        FP = 3;
    elseif strcmp(FP,'trimf','i')==0 then
        FP = 4;
    else
        error("anfislab only support gbellmf, trimf, trapmf at the moment.");
    end
end


[abc,pqr]=abcfixed(E,L,FP);
[p,ko]=size(E);
k=ko-1;

par = abc2par(abc,mem_func);

fls=newfls();
fls.ImpMethod = 'prod'
fls.AggMethod = 'sum'
fls.defuzzMethod = 'wtaver'

// L membership function per input
// k input variables

// Define empty list for variable and membership function
flsvar_def=tlist(['flsvar','name','range','mf'],"",[],list());
flsmf_def=tlist(['flsmf','name','type','par'],"","",[]);
flsvar=flsvar_def
flsmf=flsmf_def;

// Creating input nodes
for cnt_var = 1:k
    flsvar.name = 'input'+string(cnt_var);
    range_min = min(E(:,cnt_var));
    range_max = max(E(:,cnt_var));
    flsvar.range = [range_min range_max];
    fls.input(cnt_var) = flsvar;
    // Creating nodes members
    for cnt_mf = 1:L
        flsmf.name = 'in' + string(cnt_var) + 'mf'+string(cnt_mf); 
        flsmf.type = mem_func; 
        flsmf.par =par((cnt_var-1)*L+cnt_mf,:);
        fls.input(cnt_var).mf(cnt_mf) = flsmf;
    end
end


num_outmf = size(pqr,1);

flsvar=flsvar_def
flsmf=flsmf_def;

flsvar.name = 'output1';
range_min = min(E(:,$));
range_max = max(E(:,$));
flsvar.range = [range_min range_max];
fls.output(1) = flsvar;

for cnt = 1:num_outmf
    flsmf.name = 'out1mf'+string(cnt); 
    flsmf.type = 'linear'; 
    flsmf.par =pqr(cnt,:);
    fls.output(1).mf(cnt) = flsmf;
end

//fls.rule = [1 1 1 1;2 2 1 1;3 3 1 1;4 4 1 1; 5 5 1 1];

r1 = specfun_combinerepeat (1:L,k)';
//r1 = flipdim(r1,1)';
r2 = [1:L^k]';
r3 = ones(L^k,2);

fls.rule = [r1 r2 r3];



endfunction
